#include <string>
#include <iostream>
#include <fstream>
#include <vector>

using namespace std;

// nr of benchmarks
const int datasize = 12;

// holds the data of one datafile
class fdata
{
 public:
  string filename;
  int N[datasize];
  int iter[datasize];
  int init[datasize];
  float x1[datasize];
  float x2[datasize];
  float y1[datasize];
  float y2[datasize];
  float z[datasize];
};

// declarations
void parse_cmdline(int argc, char *argv[], fdata &outfile, vector<fdata> &infiles);
void parse_file(fdata &data);
void average_data(fdata &outfile, vector<fdata> &infiles);
void write_output_file(fdata &outfile);

int main(int argc, char *argv[])
{
  vector<fdata> inputfiles;
  fdata outputfile;

  parse_cmdline(argc,argv,outputfile,inputfiles);

  // parse information from all inputfiles
  for(int i=0; i<inputfiles.size(); ++i)
  {
    parse_file(inputfiles[i]);
  }

  average_data(outputfile,inputfiles);

  write_output_file(outputfile);
}

void parse_cmdline(int argc, char *argv[], fdata &outfile, vector<fdata> &infiles)
{
  if(argc<=2)
  {
    std::cout << "Arguments: " << argv[0]
              << " <outputfile> <inputfile1> <inputfile2> ...\n"
              << "at least outputfile and inputfile1 must be specified\n";
    exit(1);
  }

  // outfile
  outfile.filename = argv[1];

  // infiles
  fdata dummy;
  for(int i=2; i<argc; ++i)
  {
    dummy.filename = argv[i];
    infiles.push_back(dummy);
  }

  // DEBUG
//   cout << "outfile: " << outfile.filename << endl;
//   for(int i=0; i<infiles.size(); ++i)
//   {
//     cout << "infile " << i+1 << ": " << infiles[i].filename << endl;
//   }
}

void parse_file(fdata &data)
{
  ifstream file(data.filename.c_str());

  // discard 7 lines (comments)
  char dummy[256];
  for(int i=0; i<7; ++i)
    file.getline(dummy, 256);

  // extract the data lines
  for(int i=0; i<datasize; ++i)
  {
    file >> data.N[i] >> data.iter[i] >> data.init[i] >> data.x1[i]
         >> data.x2[i] >> data.y1[i] >> data.y2[i] >> data.z[i];
  }

  //DEBUG
//   for(int i=0; i<datasize; ++i)
//   {
//     cout << data.N[i] << endl;
//     cout << data.iter[i] << endl;
//     cout << data.init[i] << endl;
//     cout << data.x1[i] << endl;
//     cout << data.x2[i] << endl;
//     cout << data.y1[i] << endl;
//     cout << data.y2[i] << endl;
//     cout << data.z[i] << endl;
//   }
}

void average_data(fdata &outfile, vector<fdata> &infiles)
{
  // set output to 0
  for(int i=0; i<datasize; ++i)
  {
    outfile.N[i] = 0;
    outfile.iter[i] = 0;
    outfile.init[i] = 0;
    outfile.x1[i] = 0;
    outfile.x2[i] = 0;
    outfile.y1[i] = 0;
    outfile.y2[i] = 0;
    outfile.z[i] = 0;
  }

  // add up all inputs
  for(int i=0; i<datasize; ++i)
  {
    for(int j=0; j<infiles.size(); ++j)
    {
      outfile.N[i] += infiles[j].N[i];
      outfile.iter[i] += infiles[j].iter[i];
      outfile.init[i] += infiles[j].init[i];
      outfile.x1[i] += infiles[j].x1[i];
      outfile.x2[i] += infiles[j].x2[i];
      outfile.y1[i] += infiles[j].y1[i];
      outfile.y2[i] += infiles[j].y2[i];
      outfile.z[i] += infiles[j].z[i];
    }
  }

  // divide through vector size to get mean value
  for(int i=0; i<datasize; ++i)
  {
    outfile.N[i] /= infiles.size();
    outfile.iter[i] /= infiles.size();
    outfile.init[i] /= infiles.size();
    outfile.x1[i] /= infiles.size();
    outfile.x2[i] /= infiles.size();
    outfile.y1[i] /= infiles.size();
    outfile.y2[i] /= infiles.size();
    outfile.z[i] /= infiles.size();
  }

  //DEBUG
//   for(int i=0; i<datasize; ++i)
//   {
//     cout << outfile.N[i] << endl;
//     cout << outfile.iter[i] << endl;
//     cout << outfile.init[i] << endl;
//     cout << outfile.x1[i] << endl;
//     cout << outfile.x2[i] << endl;
//     cout << outfile.y1[i] << endl;
//     cout << outfile.y2[i] << endl;
//     cout << outfile.z[i] << endl;
//   }
}

void write_output_file(fdata &outfile)
{
  ofstream file(outfile.filename.c_str());

  // file header
  file << "# MathLib-Benchmark Data File\n"
       << "# AVERAGED FILE !!!\n"
       << "#\n"
       << "# TEST: AVERAGED FILE\n"
       << "# CONNECTIVITY: AVERAGED FILE\n"
       << "#\n"
       << "#<N>\t <iter>\t <init>\t <W*x>\t </el>\t <ESN>\t </el>\t <result>\n";

  // write data
  for(int i=0; i<datasize; ++i)
  {
    file << outfile.N[i] << "\t" << outfile.iter[i] << "\t"
         << outfile.init[i] << "\t"
         << outfile.x1[i] << "\t"
         << outfile.x2[i] << "\t"
         << outfile.y1[i] << "\t"
         << outfile.y2[i] << "\t"
         << outfile.z[i] << endl;
  }

  file.close();
}
