// some tools for the benchmarks
//
// Georg Holzmann, 2007

#ifndef __LIBS_BENCHMARK_TIMER_H
#define __LIBS_BENCHMARK_TIMER_H

#ifndef TYPE_FLOAT
#define TYPE double
#else
#define TYPE float
#endif

#include <stdlib.h>
#include <ctime>
#include <iostream>

//----------------------------------------------------------
// change these parameters:

int iterations = 100; // how often to compute
int N = 1000;            // Nr. of neurons
float conn = 0.2;     // Matrix connectivity
float i_conn = 0.6;   // input vector connectivity
float fb_conn = 0.7;  // feedback connectivity

//----------------------------------------------------------

// the result
TYPE y = 0;
// some constants
const TYPE alpha = 4.4, beta = 6.2;

// matrix initialization
void init_matrices();

// computaation tests
TYPE compute(int iter);
void mtx_vec_mult(int iter);

// parse cmdline arguments
void parse_cmd(int argc, char *argv[]);

// From http://flens.sourceforge.net/session2/tut4.html:
// This thing is kept very simple and adopted from Matlab.
// With t.tic() you start a timer, with t.toc() you retrieve the time
// elapsed in between these two calls.
struct timer
{
    void
    tic()
    {
        _time = std::clock();
    }

    clock_t
    toc() const
    {
        return  (std::clock() - _time);
    }

    std::clock_t _time;
};

// main function
void run_benchmarks(int argc, char *argv[])
{
  parse_cmd(argc, argv);

  timer t;

  t.tic();
  init_matrices();
  clock_t endtime0 = t.toc();
  
  t.tic();
  y = compute(iterations);
  clock_t endtime1 = t.toc();

  t.tic();
  mtx_vec_mult(iterations);
  clock_t endtime2 = t.toc();

  std::cout << N << "\t" << iterations << "\t"
     << endtime0 << "\t"
     << (float)endtime2/(float)iterations << "\t"
     << ((float)endtime2/(float)iterations)/(float)N << "\t"
     << (float)endtime1/(float)iterations << "\t"
     << ((float)endtime1/(float)iterations)/(float)N << "\t"
     << y << std::endl;


//  argv[0] << "      \t" << endtime0 << "\t"
//             << (float)endtime2/(float)iterations << "\t"
//             << ((float)endtime2/(float)iterations)/(float)N << "\t"
//             << (float)endtime1/(float)iterations << "\t"
//             << ((float)endtime1/(float)iterations)/(float)N << "\t"
//             << y << std::endl;
}

void parse_cmd(int argc, char *argv[])
{
  if(argc<=1)
  {
    std::cout << "Arguments: " << argv[0]
              << " <N> <iterations> <conn> <i_conn> <fb_conn>\n"
              << "at least N must be specified\n";
    exit(1);
  }

  if(argc > 1)
    N = atoi( argv[1] );
  if(argc > 2)
    iterations = atoi( argv[2] );
  if(argc > 3)
    conn = atof( argv[3] );
  if(argc > 4)
    i_conn = atof( argv[4] );
  if(argc > 5)
    fb_conn = atof( argv[5] );

//   std::cout << N << "\n" << iterations << "\n"
//             << conn << "\n" << i_conn << "\n" << fb_conn << "\n";
}


// random methods

//use this first function to seed the random number generator,
//call this before any of the other functions
void initrand()
{
    srand((unsigned)(time(0)));
}

//generates a psuedo-random float between 0.0 and 0.999...
TYPE randval()
{
    return rand()/(TYPE(RAND_MAX)+1);
}

#endif // __LIBS_BENCHMARK_TIMER_H
