#include <blitz/array.h>
#include <blitz/tinyvec.h>

#include "../bench-tools.h"

using namespace blitz;

typedef Array<TYPE,2> DMatrix;
typedef Array<TYPE,1> DVector;

DMatrix *W;
DVector *in;
DVector *back;
DVector *out;
DVector *x;

int main(int argc, char *argv[])
{
  run_benchmarks(argc,argv);
}

void init_matrices()
{
  W = new DMatrix(N,N);
  in = new DVector(N);
  back = new DVector(N);
  out = new DVector(2*N);
  x = new DVector(N);

  for (int i=0; i<N; ++i)
  {
    (*in)(i) = (randval()<i_conn) ? randval() : 0;
    (*back)(i) = (randval()<fb_conn) ? randval() : 0;
    (*x)(i) = randval();

    for (int j=0; j<N; ++j)
    {
      (*W)(i,j) = (randval()<conn) ? randval() : 0;
    }
  }

  for (int i=0; i<2*N; ++i)
    (*out)(i) = randval();
}

TYPE compute(int iter)
{
  firstIndex i;
  secondIndex j;

  while(iter--)
  {
    // x = W * x is quite tricky
    *x = sum( (*W)(j,i) * (*x)(i), j);
    // the rest is easy
    *x += alpha * *in;
    *x += beta * *back;

    for(int i=0; i<N; ++i)
      (*x)(i) = tanh( (*x)(i) );

    // make a temporary vector (x,x^2)
    y = sum( (*out)(Range(0,N-1)) * *x);
    for(int i=0; i<N; ++i)
      y += (*out)(N+i)*pow((*x)(i),2);
  }

  return y;
}

void mtx_vec_mult(int iter)
{
  firstIndex i;
  secondIndex j;

  while(iter--)
  {
    // x = W * x was much slower as all the compute()
    // function -> why ?
    // so now I calc in = W * x

    *in = sum( (*W)(j,i) * (*x)(i), j);
  }
}
