#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/io.hpp>
#include <boost/numeric/ublas/vector_proxy.hpp>

#include "../bench-tools.h"

using namespace boost::numeric::ublas;

typedef matrix<TYPE> GEMatrix;
typedef vector<TYPE> DEVector;

GEMatrix *W;
DEVector *in;
DEVector *back;
DEVector *out;
DEVector *x;

int main(int argc, char *argv[])
{
  run_benchmarks(argc,argv);
}

void init_matrices()
{
  W = new GEMatrix(N,N);
  in = new DEVector(N);
  back = new DEVector(N);
  out = new DEVector(2*N);
  x = new DEVector(N);

  for (int i=0; i<N; ++i)
  {
    (*in)(i) = (randval()<i_conn) ? randval() : 0;
    (*back)(i) = (randval()<fb_conn) ? randval() : 0;
    (*x)(i) = randval();

    for (int j=0; j<N; ++j)
    {
      (*W)(i,j) = (randval()<conn) ? randval() : 0;
    }
  }

  for (int i=0; i<2*N; ++i)
    (*out)(i) = randval();
}

TYPE compute(int iter)
{
  while(iter--)
  {
    *x = prod(*W,*x);
    *x += alpha * *in;
    *x += beta * *back;

    // activation function
    for(int i=0; i<N; ++i)
      (*x)(i) = tanh( (*x)(i) );

    // calculate y = [out] * [x; x^2]
    y = inner_prod( subrange(*out, 0, N), subrange(*x, 0, N) );
    for (int i=0; i<N; ++i)
      y += (*out)(N+i)*pow((*x)(i),2);
  }

  return y;
}

void mtx_vec_mult(int iter)
{
  while(iter--)
  {
    *x = prod(*W,*x);
  }
}
