#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/vector_sparse.hpp>
#include <boost/numeric/ublas/matrix_sparse.hpp>
#include <boost/numeric/ublas/io.hpp>
#include <boost/numeric/ublas/vector_proxy.hpp>

#include "../bench-tools.h"

using namespace boost::numeric::ublas;

typedef compressed_matrix<TYPE> SMatrix;
typedef compressed_vector<TYPE> SVector;
typedef vector<TYPE> DEVector;

SMatrix *W;
SVector *in;
SVector *back;
DEVector *out;
DEVector *x;

int main(int argc, char *argv[])
{
  run_benchmarks(argc,argv);
}

void init_matrices()
{
  W = new SMatrix(N,N);
  in = new SVector(N);
  back = new SVector(N);
  out = new DEVector(2*N);
  x = new DEVector(N);

  for (int i=0; i<N; ++i)
  {
    if(randval()<i_conn) (*in)(i) = randval();
    if(randval()<fb_conn) (*back)(i) = randval();
    (*x)(i) = randval();

    for (int j=0; j<N; ++j)
    {
      if(randval()<conn) (*W)(i,j) = randval();
    }
  }

  for (int i=0; i<2*N; ++i)
    (*out)(i) = randval();
}

TYPE compute(int iter)
{
  while(iter--)
  {
    *x = prod(*W,*x);
    *x += alpha * *in;
    *x += beta * *back;

    // activation function
    for(int i=0; i<N; ++i)
      (*x)(i) = tanh( (*x)(i) );

    // calculate y = [out] * [x; x^2]
    y = inner_prod( subrange(*out, 0, N), subrange(*x, 0, N) );
    for (int i=0; i<N; ++i)
      y += (*out)(N+i)*pow((*x)(i),2);
  }

  return y;
}


void mtx_vec_mult(int iter)
{
  while(iter--)
  {
    *x = prod(*W,*x);
  }
}
