#include <flens/flens.h>
#include <math.h>
#include "../bench-tools.h"

using namespace flens;
using namespace std;

typedef GeMatrix<FullStorage<TYPE, ColMajor> > GEMatrix;
typedef DenseVector<Array<TYPE> > DEVector;

GEMatrix *W;
DEVector *in;
DEVector *back;
DEVector *out;
DEVector *x;

int main(int argc, char *argv[])
{
  run_benchmarks(argc,argv);
}

void init_matrices()
{
  W = new GEMatrix(N,N);
  in = new DEVector(N);
  back = new DEVector(N);
  out = new DEVector(2*N);
  x = new DEVector(N);

  for (int i=1; i<=N; ++i)
  {
    if(randval()<i_conn) (*in)(i) = randval();
    if(randval()<fb_conn) (*back)(i) = randval();
    (*x)(i) = randval();

    for (int j=1; j<=N; ++j)
    {
      if(randval()<conn) (*W)(i,j) = randval();
    }
  }

  for (int i=1; i<=2*N; ++i)
    (*out)(i) = randval();
}

TYPE compute(int iter)
{
  // temporary objects
  DEVector t;
  DEVector::View out1 = (*out)(_(1,N)), out2 = (*out)(_(N+1,2*N));

  while(iter--)
  {
    t = *x; // temporary object needed for BLAS
    *x = alpha * *in + *W * t + beta * *back;

    // activation function
    for(int i=1; i<=N; ++i)
      (*x)(i) = tanh( (*x)(i) );

    // calculate y = [out] * [x; x^2]
    y = dot(out1, *x);
    for (int i=1; i<=N; ++i)
      y += out2(i)*pow((*x)(i),2);
  }

  return y;
}

void mtx_vec_mult(int iter)
{
  DEVector::View t1 = (*x)(_(1,N));

  while(iter--)
  {
    *x = *W * t1;
  }
}
