#include "gmm/gmm.h"

#include "../bench-tools.h"

typedef gmm::csr_matrix<TYPE,1> SPMatrix;
typedef gmm::rsvector<TYPE> SPVector;
typedef std::vector<TYPE> DVector;

SPMatrix *W;
SPVector *in;
SPVector *back;
DVector *out;
DVector *x;

int main(int argc, char *argv[])
{
  run_benchmarks(argc,argv);
}

void init_matrices()
{
  W = new SPMatrix(N,N);
  in = new SPVector(N);
  back = new SPVector(N);
  out = new DVector(2*N);
  x = new DVector(N);

  // we need temporary writeable sparse objects:
  gmm::row_matrix< gmm::wsvector<double> > W1(N,N);
  gmm::wsvector<double> in1(N);
  gmm::wsvector<double> back1(N);

  for (int i=0; i<N; ++i)
  {
   if(randval()<i_conn) in1[i] = randval();
   if(randval()<fb_conn) back1[i] = randval();
   (*x)[i] = randval();

    for (int j=0; j<N; ++j)
    {
      if (randval()<conn) W1(i,j) = randval();
    }
  }

 for (int i=0; i<2*N; ++i)
   (*out)[i] = randval();

  // now copy to the original sparse matrix
  gmm::copy(W1, *W);
  gmm::copy(in1, *in);
  gmm::copy(back1, *back);
}

TYPE compute(int iter)
{
  // temporary objects
  DVector t1(N);
  DVector t2(2*N);

  while(iter--)
  {
    gmm::copy(*x,t1);
    gmm::mult(*W, t1, *x);
    gmm::add(*x, gmm::scaled(*in, alpha), *x);
    gmm::add(*x, gmm::scaled(*back, beta), *x);

    // activation function
    for(int i=0; i<N; ++i)
      (*x)[i] = tanh( (*x)[i] );

    // calculate y = [out] * [x; x^2]
    y = gmm::vect_sp(gmm::sub_vector(*out, gmm::sub_interval(0, N)), *x);
    for (int i=0; i<N; ++i)
      y += (*out)[N+i]*pow((*x)[i],2);
  }

  return y;
}

void mtx_vec_mult(int iter)
{
  DVector t1(N);

  while(iter--)
  {
    gmm::copy(*x,t1);
    gmm::mult(*W, t1, *x);
  }
}
