#include "mtl/matrix.h"
#include <mtl/mtl.h>
#include <mtl/utils.h>

#include "../bench-tools.h"

using namespace mtl;

typedef matrix<TYPE, rectangle<>, dense<>, 
               row_major>::type  DMatrix;
typedef dense1D<TYPE> DVector;

DMatrix *W;
DVector *in;
DVector *back;
DVector *out;
DVector *x;

int main(int argc, char *argv[])
{
  run_benchmarks(argc,argv);
}

void init_matrices()
{
  W = new DMatrix(N,N);
  in = new DVector(N);
  back = new DVector(N);
  out = new DVector(2*N);
  x = new DVector(N);

  for (int i=0; i<N; ++i)
  {
    (*in)[i] = (randval()<i_conn) ? randval() : 0;
    (*back)[i] = (randval()<fb_conn) ? randval() : 0;
    (*x)[i] = randval();

    for (int j=0; j<N; ++j)
    {
      (*W)(i,j) = (randval()<conn) ? randval() : 0;
    }
  }

  for (int i=0; i<2*N; ++i)
    (*out)[i] = randval();
}

TYPE compute(int iter)
{
  // temporary object
  DVector t1(N);

  while(iter--)
  {
    copy(*x,t1);
    mult(*W, t1, *x);
    add(*x, scaled(*in, alpha), *x);
    add(*x, scaled(*back, beta), *x);

    // activation function
    for(int i=0; i<N; ++i)
      (*x)[i] = tanh( (*x)[i] );

    y = dot( (*out)(0,N), *x);
    for (int i=0; i<N; ++i)
      y += (*out)[N+i]*pow((*x)[i],2);
  }

  return y;
}

void mtx_vec_mult(int iter)
{
  DVector t1(N);

  while(iter--)
  {
    copy(*x,t1);
    mult(*W, t1, *x);
  }
}
