#!/bin/sh
#
# Georg Holzmann, 2007
#
# plots math-library benchmarks using gnuplot
# USAGE: type ./plot.sh
#
# NOTE: it also stores the picture into the plots
#       subfolder


#---------------------------------------------
# test input arguments
if [ $# = 0 ]
then
  echo "USAGE:"
  echo "$0 <TYPE> <PRECISION> <CONN> <VERSION>"
  echo "\t<TYPE>: 0 = mtx*vec multiplication, 1 = ESN operation, 3 = initialization"
  echo "\t<PRECISION>: float or double"
  echo "\t<CONN>: connectivity, e.g. 0.2"
  echo "\t<VERSION>: ending datafiles, e.g. 1 in 'conn0.2-1.dat'"
  echo "\t<SHOW>: if 'off', then only plotting to file (standard=on)"
  exit
fi

# type of the plot
case $1 in
"1") echo "plotting mtx*vec mult"; ROW="1:5" ;;
"2") echo "plotting ESN operation"; ROW="1:7" ;;
"3") echo "plotting initialization"; ROW="1:3" ;;
*) echo "wrong TYPE parameters"; exit ;;
esac

# precision
case $2 in
"float") echo "using floats"; PREC="float" ;;
"double") echo "using doubles"; PREC="double" ;;
*) echo "wrong PRECISION parameters"; exit ;;
esac

# version string
CONN=$3
VSTR=$4
VERSION="conn${CONN}-${VSTR}.dat"
echo "trying to plot DIR/LIB_${VERSION} files ..."

case $5 in
"off") PLOTTING="off" ;;
*) PLOTTING="on" ;;
esac

# temp plot file
PLOTFILE="plotit.p"


#---------------------------------------------
# writing gnuplot properties
make_header()
{
  case $1 in
  "1")
    echo "set title 'Matrix-Vector Multiplication x=W*x, ${PREC}, W-Connectivity=${CONN}'" > $PLOTFILE
    echo "set ylabel 'CPU ticks per element'" >> $PLOTFILE ;;
  "2")
    echo "set title 'ESN like operation, ${PREC}, W-Connectivity=${CONN}'" > $PLOTFILE
    echo "set ylabel 'CPU ticks per element'" >> $PLOTFILE ;;
  "3")
    echo "set title 'Initialization, ${PREC}, W-Connectivity=${CONN}'" > $PLOTFILE
    echo "set ylabel 'CPU ticks'" >> $PLOTFILE ;;
  esac

   echo "set key left top" >> $PLOTFILE
   echo "set xlabel 'Matrix/Vector Size'" >> $PLOTFILE
   echo "set logscale" >> $PLOTFILE
   echo "set xtics (10,30,50,70,85,100,200,300,500,1000,2000,5000,10000)" >> $PLOTFILE
}

# the first plot
first_plot()
{
  DATA="${DIR}/${LIB}_${SP}_${PREC}-${VERSION}"

  echo "plot '${DIR}/${LIB}_${SP}_${PREC}-${VERSION}' using ${ROW} title '${LIB}_${SP}' with linespoints , \\" >> $PLOTFILE
}

# one plot
draw_plot()
{
  DATA="${DIR}/${LIB}_${SP}_${PREC}-${VERSION}"

  echo "'${DIR}/${LIB}_${SP}_${PREC}-${VERSION}' using ${ROW} title '${LIB}_${SP}' with linespoints , \\" >> $PLOTFILE
}

# last plot
last_plot()
{
  DATA="${DIR}/${LIB}_${SP}_${PREC}-${VERSION}"

  echo "'${DIR}/${LIB}_${SP}_${PREC}-${VERSION}' using ${ROW} title '${LIB}_${SP}' with linespoints" >> $PLOTFILE
}

# export plot to pdf
export_to_jpeg()
{
  case $1 in
  "1")
    PLOT="plots/MtxVecMult_${PREC}_conn${CONN}_${VSTR}" ;;
  "2")
    PLOT="plots/ESNOperation_${PREC}_conn${CONN}_${VSTR}" ;;
  "3")
    PLOT="plots/Init_${PREC}_conn${CONN}_${VSTR}" ;;
  esac

  echo "set terminal jpeg medium size 1200,900" >> $PLOTFILE
  echo "set output '${PLOT}_large.jpg'" >> $PLOTFILE
  echo "replot" >> $PLOTFILE

  echo "set terminal jpeg small size 680,510" >> $PLOTFILE
  echo "set output '${PLOT}_small.jpg'" >> $PLOTFILE
  echo "replot" >> $PLOTFILE

  # ZOOM interesting area

  echo "unset logscale; set logscale x" >> $PLOTFILE
  echo "set yrange [0:0.7]" >> $PLOTFILE
  echo "set xrange [30:500]" >> $PLOTFILE
  echo "set xtics (30,50,70,85,100,200,300,500)" >> $PLOTFILE

  echo "set terminal jpeg medium size 1200,900" >> $PLOTFILE
  echo "set output '${PLOT}_large_zoom.jpg'" >> $PLOTFILE
  echo "replot" >> $PLOTFILE

  echo "set terminal jpeg small size 680,510" >> $PLOTFILE
  echo "set output '${PLOT}_small_zoom.jpg'" >> $PLOTFILE
  echo "replot" >> $PLOTFILE
}


#---------------------------------------------
# MAIN PROGRAM

make_header $1

# ploting matrix-vector multiplication

LIB=blitz
DIR=blitz++
SP=nonsparse
first_plot

if [ $PREC = double ]
then
  LIB=newmat
  DIR=newmat
  SP=nonsparse
  draw_plot
fi

LIB=boost
DIR=boost-ublas
SP=nonsparse
draw_plot

SP=sparse
draw_plot

LIB=flens
DIR=flens
SP=nonsparse
draw_plot

SP=sparse
draw_plot

LIB=gmm
DIR=gmm++
SP=nonsparse
draw_plot

SP=sparse
draw_plot

LIB=mtl
DIR=mtl
SP=nonsparse
draw_plot

SP=sparse
last_plot

# plot the jpegs
export_to_jpeg $1

if [ $PLOTTING = "on" ]
then gnuplot -persist $PLOTFILE
else gnuplot $PLOTFILE
fi

# cleanup
rm -f $PLOTFILE