#!/bin/sh
#
# Georg Holzmann, 2007
#
# this script runs the benchmark for one lib
# USAGE:
# type run_benchmark.sh for help


# test input arguments
if [ $# = 0 ]
then
  echo "USAGE:"
  echo "$0 <program-name> <version-string> <conn>"
  echo "\t<program-name>: e.g. newmat/newmat_nonsparse_double"
  echo "\t<version-string>: string for the datafile name"
  echo "\t<conn>: connectivity of the Matrix (0..1)"
  exit
fi

# parse arguments
PROG=$1
VERS=$2
if [ $# = 3 ]
then CONN=$3
else CONN="0.2"
fi

DATAFILE="$PROG-conn$CONN-$VERS.dat"

# writing file header
echo "# MathLib-Benchmark Data File" > $DATAFILE
echo "# generated on `date`" >> $DATAFILE
echo "#" >> $DATAFILE
echo "# TEST: $PROG" >> $DATAFILE
echo "# CONNECTIVITY: $CONN" >> $DATAFILE
echo "#" >> $DATAFILE
echo "#<N>\t <iter>\t <init>\t <W*x>\t </el>\t <ESN>\t </el>\t <result>" >> $DATAFILE


#---------------------------------------------
# executing benchmarks

echo "---------------------------------"
echo "executing: \n" $PROG
echo "results in:\n" $DATAFILE
echo ""

NR=10
ITER=200000
CMD="$PROG $NR $ITER $CONN"
echo "$CMD ..."
$CMD >> $DATAFILE

NR=30
ITER=100000
CMD="$PROG $NR $ITER $CONN"
echo "$CMD ..."
$CMD >> $DATAFILE

NR=50
ITER=100000
CMD="$PROG $NR $ITER $CONN"
echo "$CMD ..."
$CMD >> $DATAFILE

NR=70
ITER=50000
CMD="$PROG $NR $ITER $CONN"
echo "$CMD ..."
$CMD >> $DATAFILE

NR=85
ITER=50000
CMD="$PROG $NR $ITER $CONN"
echo "$CMD ..."
$CMD >> $DATAFILE

NR=100
ITER=10000
CMD="$PROG $NR $ITER $CONN"
echo "$CMD ..."
$CMD >> $DATAFILE

NR=200
ITER=10000
CMD="$PROG $NR $ITER $CONN"
echo "$CMD ..."
$CMD >> $DATAFILE

NR=300
ITER=6000
CMD="$PROG $NR $ITER $CONN"
echo "$CMD ..."
$CMD >> $DATAFILE

NR=500
ITER=3000
CMD="$PROG $NR $ITER $CONN"
echo "$CMD ..."
$CMD >> $DATAFILE

NR=1000
ITER=1000
CMD="$PROG $NR $ITER $CONN"
echo "$CMD ..."
$CMD >> $DATAFILE

NR=2000
ITER=250
CMD="$PROG $NR $ITER $CONN"
echo "$CMD ..."
$CMD >> $DATAFILE

NR=5000
ITER=50
CMD="$PROG $NR $ITER $CONN"
echo "$CMD ..."
$CMD >> $DATAFILE

# NR=10000
# ITER=1
# CMD="$PROG $NR $ITER $CONN"
# echo "$CMD ..."
# $CMD >> $DATAFILE

echo "... finished"
